/*
 * Author: Daniel Nitecki
 * Date: 11-10-2012 23:59:34
 * 
 * Description:
 * This program generates the specific waveforms that control Syma helicopter movements.
 */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f4_discovery.h"

/** @addtogroup STM32F4_Discovery_Peripheral_Examples
 * @{
 */

/** @addtogroup TIM_PWM_Output
 * @{
 */

/* Private typedef -----------------------------------------------------------*/
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
TIM_OCInitTypeDef TIM_OCInitStructure;


/* Private define ------------------------------------------------------------*/
#define RAM_BASE 0x20000000
#define RAM_BB_BASE 0x22000000
#define Var_ResetBit_BB(VarAddr, BitNumber) (*(vu32 *) (RAM_BB_BASE | ((VarAddr - RAM_BASE) << 5) | ((BitNumber) << 2)) = 0)
#define Var_SetBit_BB(VarAddr, BitNumber) (*(vu32 *) (RAM_BB_BASE | ((VarAddr - RAM_BASE) << 5) | ((BitNumber) << 2)) = 1)
#define Var_GetBit_BB(VarAddr, BitNumber) (*(vu32 *) (RAM_BB_BASE | ((VarAddr - RAM_BASE) << 5) | ((BitNumber) << 2)))

#define varClrBit(var,bit) (Var_ResetBit_BB((u32)&var,bit))
#define varSetBit(var,bit) (Var_SetBit_BB((u32)&var,bit))
#define varGetBit(var,bit) (Var_GetBit_BB((u32)&var,bit))
//-------------
#define st_Start 0
#define st_Yaw 1
#define st_Pitch 2
#define st_Throttle 3
#define st_Adjust 4
#define st_Stop 5

/* Private macro -------------------------------------------------------------*/

/* Private variables ---------------------------------------------------------*/
// Helicopter
uint16_t PrescalerValue = 0;

uint8_t YAW_value = 0b00111111;//0b_0_0111000; //63 center
uint8_t PITCH_value = 0b00111111;//0b_0_0111111; //63 center
uint8_t THROTTLE_value = 0b00000000;//0b_0_1010111; //0 = no throttle, 127 = max throttle
uint8_t ADJUST_value = 0b00100110;//0b_0_0100110;

uint8_t actual_cell = 7;

uint8_t cState = st_Start;
uint8_t nState = st_Start;

// Accelerometer
uint8_t Buffer[6];
uint8_t Buffer_Throttle = 0b00000000;
uint8_t Buffer_Yaw = 0b00111111;//0b_0_0111111; //63 center
const uint8_t MoveRangeValue = 50;
__IO uint32_t TimingDelay = 0;
__IO int8_t XOffset;
__IO int8_t YOffset;
__IO int8_t ZOffset;

__IO uint8_t SingleClickDetect = 0x00;
extern uint8_t ClickReg;

/* Private function prototypes -----------------------------------------------*/
void ThrottleLinkConfig(void);
void TIM_Config(void);

void INTTIM_Config(void);

void LED_Config(void);

void SetCurrentPWMOutput(uint8_t ValueToSend);

/* Private functions ---------------------------------------------------------*/

void TIM2_IRQHandler(void)
{
//|'''|_______|^
	if (TIM_GetITStatus(TIM2, TIM_IT_Update) != RESET)
	{
switch (cState)
{
	case st_Start:
		nState = st_Yaw;
		TIM_TimeBaseStructure.TIM_Period = 3200*2;
		TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
		TIM2->CCR1 = 1600*2;
		TIM2->CNT = 3200*2;
		cState = nState;
		break;
		
	case st_Yaw:
		nState = st_Pitch;
		Buffer_Yaw = (-1*Buffer[2]);
		if (((int8_t)Buffer_Yaw)>MoveRangeValue)
		{
			YAW_value = 127;
		}
		else if (((int8_t)Buffer_Yaw)<(-1*MoveRangeValue))
		{
			YAW_value = 0;
		}
		else if ((((int8_t)Buffer_Yaw)<(MoveRangeValue/2)) && (((int8_t)Buffer_Yaw)>(-1*MoveRangeValue/2)))`
		{
			YAW_value = 63;	// engine off: 63 ~= 127/2
		}
		else
		{
			if((((int8_t)Buffer_Yaw+MoveRangeValue)*127/(2*MoveRangeValue)) > 127)
			{
				YAW_value = 127;
			}
			else
			{
				YAW_value = ((int8_t)Buffer_Yaw+MoveRangeValue)*127/(2*MoveRangeValue);
			}
		}
		SetCurrentPWMOutput(YAW_value);
		break;
		
	case st_Pitch:
		nState = st_Throttle;
		if (((int8_t)Buffer[0])>MoveRangeValue)
		{
			PITCH_value = 127;
		}
		else if (((int8_t)Buffer[0])<(-1*MoveRangeValue))
		{
			PITCH_value = 0;
		}
		else if ((((int8_t)Buffer[0])<(MoveRangeValue/3.5)) && (((int8_t)Buffer[0])>(-1*MoveRangeValue/3.5)))
		{
			PITCH_value = 63;	// engine off: 63 ~= 127/2
		}
		else
		{
			if((((int8_t)Buffer[0]+MoveRangeValue)*127/(2*MoveRangeValue)) > 127)
			{
				PITCH_value = 127;
			}
			else
			{
				PITCH_value = ((int8_t)Buffer[0]+MoveRangeValue)*127/(2*MoveRangeValue);
			}
		}
		SetCurrentPWMOutput(PITCH_value);
		break;
		
	case st_Throttle:
		nState = st_Adjust;
		uint8_t i;
		for(i = 8; i < 16; ++i)
		{
			if(GPIO_ReadInputDataBit(GPIOE, (uint16_t)1<<i)==(uint8_t)Bit_SET)
			{
				varSetBit(Buffer_Throttle,i-8);
			}
			else
			{
				varClrBit(Buffer_Throttle,i-8);
			}
		}
		
		if (((int8_t)Buffer_Throttle)<0)
		{
			THROTTLE_value = 0;
		}
		else
		{
			if((((int8_t)Buffer_Throttle)*127/MoveRangeValue) > 127)
			{
				THROTTLE_value = 127;
			}
			else
			{
				THROTTLE_value = ((int8_t)Buffer_Throttle)*127/MoveRangeValue;
			}
		}
		SetCurrentPWMOutput(THROTTLE_value);
		break;
	
	case st_Adjust:
		nState = st_Stop;
		SetCurrentPWMOutput(ADJUST_value);
		break;
		
	case st_Stop:
		nState = st_Start;
		TIM_TimeBaseStructure.TIM_Period = 32240*2;
		TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
		TIM2->CCR1 = 240*2;
		TIM2->CNT = 32240*2;
		cState = nState;
		break;
			
	default:
		break;
}
		TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
		TIM_ClearITPendingBit(TIM2, TIM_IT_CC1);
		
		GPIO_SetBits(GPIOD, GPIO_Pin_13);

		GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_TIM3);
		//TIM_TimeBaseStructure.TIM_Period = 1440;
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
	}
//|'''|^_______|
	if (TIM_GetITStatus(TIM2, TIM_IT_CC1) != RESET)
	{
		/*switch (cState)
		{
			case st_Start:
				cState = nState;
				break;
				
			case st_Stop:
				cState = nState;
				break;
				
			default:
				break;
		}*/
		TIM_ClearITPendingBit(TIM2, TIM_IT_CC1);
		TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
		
		GPIO_ResetBits(GPIOD, GPIO_Pin_13);

		GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_RTC_50Hz);
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, DISABLE);
	}
}

/**
 * @brief  Main program
 * @param  None
 * @retval None
 */
int main(void)
{
	/*!< At this stage the microcontroller clock setting is already configured,
	 this is done through SystemInit() function which is called from startup
	 file (startup_stm32f4xx.s) before to branch to application main.
	 To reconfigure the default setting of SystemInit() function, refer to
	 system_stm32f4xx.c file
	 */
	  uint8_t ctrl = 0;
	  
	  LIS302DL_InitTypeDef  LIS302DL_InitStruct;
	  LIS302DL_InterruptConfigTypeDef LIS302DL_InterruptStruct;  
	  
	  /* SysTick end of count event each 10ms */
	  SysTick_Config(SystemCoreClock/100);
	  
	  /* Set configuration of LIS302DL*/
	  LIS302DL_InitStruct.Power_Mode = LIS302DL_LOWPOWERMODE_ACTIVE;
	  LIS302DL_InitStruct.Output_DataRate = LIS302DL_DATARATE_100;
	  LIS302DL_InitStruct.Axes_Enable = LIS302DL_X_ENABLE | LIS302DL_Y_ENABLE | LIS302DL_Z_ENABLE;
	  LIS302DL_InitStruct.Full_Scale = LIS302DL_FULLSCALE_2_3;
	  LIS302DL_InitStruct.Self_Test = LIS302DL_SELFTEST_NORMAL;
	  LIS302DL_Init(&LIS302DL_InitStruct);
	    
	  /* Set configuration of Internal High Pass Filter of LIS302DL*/
	  LIS302DL_InterruptStruct.Latch_Request = LIS302DL_INTERRUPTREQUEST_LATCHED;
	  LIS302DL_InterruptStruct.SingleClick_Axes = LIS302DL_CLICKINTERRUPT_Z_ENABLE;
	  LIS302DL_InterruptStruct.DoubleClick_Axes = LIS302DL_DOUBLECLICKINTERRUPT_Z_ENABLE;
	  LIS302DL_InterruptConfig(&LIS302DL_InterruptStruct);

	  /* Required delay for the MEMS Accelerometer: Turn-on time = 3/Output data Rate
	                                                             = 3/100 = 30ms */
	  Delay(30);
	  
	  /* Configure Interrupt control register: enable Click interrupt1 */
	  ctrl = 0x07;
	  LIS302DL_Write(&ctrl, LIS302DL_CTRL_REG3_ADDR, 1);
	  
	  /* Enable Interrupt generation on click/double click on Z axis */
	  ctrl = 0x70;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_CFG_REG_ADDR, 1);
	  
	  /* Configure Click Threshold on X/Y axis (10 x 0.5g) */
	  ctrl = 0xAA;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_THSY_X_REG_ADDR, 1);
	  
	  /* Configure Click Threshold on Z axis (10 x 0.5g) */
	  ctrl = 0x0A;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_THSZ_REG_ADDR, 1);
	  
	  /* Configure Time Limit */
	  ctrl = 0x03;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_TIMELIMIT_REG_ADDR, 1);
	    
	  /* Configure Latency */
	  ctrl = 0x7F;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_LATENCY_REG_ADDR, 1);
	  
	  /* Configure Click Window */
	  ctrl = 0x7F;
	  LIS302DL_Write(&ctrl, LIS302DL_CLICK_WINDOW_REG_ADDR, 1);
	  
	/* TIM Configuration */
	TIM_Config();
	
	LIS302DL_Read(Buffer, LIS302DL_OUT_X_ADDR, 6);
	                  
	  XOffset = Buffer[0];
	  YOffset = Buffer[2];
	  
	INTTIM_Config();
	/* LED Configuration */
	LED_Config();
	ThrottleLinkConfig();

	while (1)
	{
	}
}

/**
 * @brief  Configure the TIM3 Output Channels.
 * @param  None
 * @retval None
 */
void TIM_Config(void) //blue
{
	GPIO_InitTypeDef GPIO_InitStructure;
	
	/* TIM3 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	/* GPIOC and GPIOB clock enable */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

	/* GPIOC Configuration: TIM3 CH1 (PC6) and TIM3 CH2 (PC7) */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	/* Connect TIM3 pins to AF2 */
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_TIM3);

	/* -----------------------------------------------------------------------
	 TIM3 Configuration: generate 4 PWM signals with 4 different duty cycles.

	 In this example TIM3 input clock (TIM3CLK) is set to 2 * APB1 clock (PCLK1),
	 since APB1 prescaler is different from 1.
	 TIM3CLK = 2 * PCLK1
	 PCLK1 = HCLK / 4
	 => TIM3CLK = HCLK / 2 = SystemCoreClock /2

	 To get TIM3 counter clock at 28 MHz, the prescaler is computed as follows:
	 Prescaler = (TIM3CLK / TIM3 counter clock) - 1
	 Prescaler = ((SystemCoreClock /2) /28 MHz) - 1

	 To get TIM3 output clock at 30 KHz, the period (ARR)) is computed as follows:
	 ARR = (TIM3 counter clock / TIM3 output clock) - 1
	 = 665

	 TIM3 Channel1 duty cycle = (TIM3_CCR1/ TIM3_ARR)* 100 = 50%
	 TIM3 Channel2 duty cycle = (TIM3_CCR2/ TIM3_ARR)* 100 = 37.5%
	 TIM3 Channel3 duty cycle = (TIM3_CCR3/ TIM3_ARR)* 100 = 25%
	 TIM3 Channel4 duty cycle = (TIM3_CCR4/ TIM3_ARR)* 100 = 12.5%

	 Note:
	 SystemCoreClock variable holds HCLK frequency and is defined in system_stm32f4xx.c file.
	 Each time the core clock (HCLK) changes, user had to call SystemCoreClockUpdate()
	 function to update SystemCoreClock variable value. Otherwise, any configuration
	 based on this variable will be incorrect.
	 ----------------------------------------------------------------------- */

	/* Compute the prescaler value */
	PrescalerValue = (uint16_t)((SystemCoreClock) / 1333333) - 1;

	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 34;
	TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);

	/* PWM1 Mode configuration: Channel1 */
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Toggle; //TIM_OCMode_PWM1;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	TIM_OC1Init(TIM3, &TIM_OCInitStructure);

	TIM_OC1PreloadConfig(TIM3, TIM_OCPreload_Enable);

	TIM_ARRPreloadConfig(TIM3, ENABLE);

	/* TIM3 enable counter */
	TIM_Cmd(TIM3, ENABLE);
}

void Delay(__IO uint32_t nTime)
{ 
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
  if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

/**
  * @brief  MEMS accelerometre management of the timeout situation.
  * @param  None.
  * @retval None.
  */
uint32_t LIS302DL_TIMEOUT_UserCallback(void)
{
  /* MEMS Accelerometer Timeout error occured */
  while (1)
  {   
  }
}

/**
 * @brief  Setup an interval timer
 * @param  None
 * @retval None
 */
void INTTIM_Config(void) //yellow
{
	NVIC_InitTypeDef NVIC_InitStructure;
	/* Enable the TIM2 global Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	//---------------------------------------

	/* TIM2 clock enable */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 3200*2;
	TIM_TimeBaseStructure.TIM_Prescaler = 9;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;

	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

	/* PWM1 Mode configuration: Channel1 */
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = 0;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;

	TIM_OC1Init(TIM2, &TIM_OCInitStructure);

	TIM_OC1PreloadConfig(TIM2, TIM_OCPreload_Enable);

	TIM_ARRPreloadConfig(TIM2, ENABLE);
	//---------------------------------------

	/* TIM2 clock enable
	 RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
	 Time base configuration
	 TIM_TimeBaseStructure.TIM_Period = 10000 - 1;  // 1 MHz down to 1 KHz (1 ms)
	 TIM_TimeBaseStructure.TIM_Prescaler = 80 - 1; // 24 MHz Clock down to 1 MHz (adjust per your clock)
	 TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	 TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	 TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);*/
	/* TIM IT enable */
	TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
	TIM_ITConfig(TIM2, TIM_IT_CC1, ENABLE);
	/* TIM2 enable counter */
	TIM_Cmd(TIM2, ENABLE);

	TIM2->CCR1 = 1600*2;
}

void SetCurrentPWMOutput(uint8_t ValueToSend)
{
	if (actual_cell==0)
	{
		if (varGetBit(ValueToSend,actual_cell)==0)
		{
			TIM_TimeBaseStructure.TIM_Period = 480*2;
			TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
			TIM2->CCR1 = 240*2;
			TIM2->CNT = 0;
		}
		else
		{
			TIM_TimeBaseStructure.TIM_Period = 720*2;
			TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
			TIM2->CCR1 = 240*2;
			TIM2->CNT = 0;
		}
		actual_cell = 7;
		cState = nState;
	}
	else
	{
		if (varGetBit(ValueToSend,actual_cell)==0)
		{
			TIM_TimeBaseStructure.TIM_Period = 480*2;
			TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
			TIM2->CCR1 = 240*2;
			TIM2->CNT = 480*2;
		}
		else
		{
			TIM_TimeBaseStructure.TIM_Period = 720*2;
			TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
			TIM2->CCR1 = 240*2;
			TIM2->CNT = 720*2;
		}
		actual_cell = actual_cell - 1;
	}
}

void LED_Config(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	/* GPIOD Periph clock enable */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

	/* Configure PD12, PD13, PD14 and PD15 in output pushpull mode */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14
			| GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
}

void ThrottleLinkConfig(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	
	/* GPIOA Periph clock enable */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOE, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
	GPIO_Init(GPIOE, &GPIO_InitStructure);
}


#ifdef  USE_FULL_ASSERT

/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t* file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	while (1)
	{}
}
#endif

/**
 * @}
 */

/**
 * @}
 */

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
